unit uodge_scandir;


{
  
   Project  : ODGE
   Unit Name: uodge_updgauge

 /--------------------------------------------------------------\
 | Purpose    : Scan a disk from given folder                   |
 | Author     : Olivier DAHAN odahan@cybercable.fr              |
 |              http://perso.cybercable.fr/stargate             |
 +--------------------------------------------------------------+
 | History    : V 1.5                                           |
 | Created on : 16/04/98 02:10:15                               |
 | Modified on: jun-22-99                                       |
 \--------------------------------------------------------------/
}

{principle :                                                       }
{Only files corresponding to the filter are returned (ie: *.bmp).               }
{Only files having attributs "attributs" are returned. Use "FaAnyFile" to get all files}
{If "rcursif" is set to true, subdirectories are also scanned.                 }
{When a file corresponding to filter is found, proc PAdeclencherFichier is called }
{When a directory is found proc PAdeclencherDossier     }
{By default, those procs do nothing. You must supply yours as below:

procedure MyProc(FileName:String;objet:Tobject);
begin
    ..here, your code. Found file is "Filename"
end
before executing ScruteDossier, you must assign your proc like this: PAdeclencherFichier:=Myproc;
To hold folders, you can do the same with PADeclencheDossier}


interface

uses   Windows, Messages, SysUtils, Classes, Controls, Forms;

type TFDeclenche=procedure(NomFic:String;objet:Tobject);

var PAdeclencherFichier : TFDeclenche; // File
    PAdeclencherDossier : TFDeclenche; // Folder
    PACallingObject : Tobject = nil; // object ref

// Scan directory
Procedure ScruteDossier(
           Dossier:string; // starting folder
           filtre:string; // filter
           attributs:integer; // attributs
           recursif:boolean); // recursive call (e.g. scanning subdirectories)

implementation


// dummy proc if no proc is assigned by the called
Procedure vide(Nom:String;objet:Tobject);
begin
end;

// file scanning
Procedure ScruteFichier(Dossier:string;filtre:string;Attributs:integer);
var FichierTrouve:string;
    Resultat:Integer;
    SearchRec:TSearchRec;
begin
  If Dossier[length(Dossier)]='\' then Dossier:=copy(Dossier,1,length(Dossier)-1);
  Resultat:=FindFirst(Dossier+'\'+filtre,Attributs,SearchRec);
  while Resultat=0 do
  begin
    Application.ProcessMessages; // let a chance to the app to see messages
     if ((SearchRec.Attr and faDirectory)<=0) then // we found a file (and a folder name)
    begin
      FichierTrouve:=Dossier+'\'+SearchRec.Name;
      PAdeclencherFichier(Dossier+'\'+SearchRec.Name,PACallingObject);// File found : call the user callback proc
    end;
    Resultat:=FindNext(SearchRec);
  end;
  FindClose(SearchRec);// freeing memory
end;

// main proc
Procedure ScruteDossier(Dossier:string;filtre:string;attributs:integer;recursif:boolean);
var DossierTrouve:string;
    Resultat:Integer;
    SearchRec:TSearchRec;
begin
  If Dossier[length(Dossier)]='\' then Dossier:=copy(Dossier,1,length(Dossier)-1); // strip off final '\'
  ScruteFichier(Dossier,filtre,attributs); //find files in current folder
  Resultat:=FindFirst(Dossier+'\'+'*.*',FaDirectory,SearchRec); // find the 1st subdir of current dir
  while Resultat=0 do                                           // SearchRec has all info about found dir
  begin
   if (SearchRec.Name<>'.') and (SearchRec.Name<>'..')
         and ((SearchRec.Attr and faDirectory)>0)then // detectes if we found a file or a folder
    begin
      DossierTrouve:=Dossier+'\'+SearchRec.Name; // to get the full name of folder
      PADeclencherDossier(DossierTrouve,PACallingObject);// Folder found, call the user callback proc
      if recursif then
      begin
        ScruteDossier(DossierTrouve,filtre,attributs,recursif);// recursive call from current folder
      end;
      Application.ProcessMessages; // lets a chance to caller to see messages
    end;
    Resultat:=FindNext(SearchRec); // find next subdirs
  end;
  FindClose(SearchRec);// freeing memory
end;


Initialization
  PAdeclencherFichier:=vide;
  PAdeclencherDossier:=vide;
end.



Sample

unit Unit1;
interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Buttons,ScruteFichiers;
type
  TForm1 = class(TForm)
    Edit1: TEdit;
    Label1: TLabel;
    Button1: TButton;
    Memo1: TMemo;
    Memo2: TMemo;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Edit2: TEdit;
    CheckBox8: TCheckBox;
    BitBtn1: TBitBtn;
    procedure Button1Click(Sender: TObject);
  end;

var
  Form1: TForm1;

procedure PourFichier(NomFichier:String;objet:Tobject); // for one file
procedure PourDossier(NomDossier:String;objet:Tobject); // for one folder

implementation

{$R *.DFM}

procedure PourFichier(NomFichier:String;objet:Tobject); // a file is found
begin
 Form1.Memo1.Lines.Add(NomFichier);// add found file name in memo1
end;

procedure PourDossier(NomDossier:String;objet:Tobject); // a folder is found
begin
 Form1.Memo2.Lines.Add(NomDossier); // add folder name in memo2
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  PADeclencherFichier:=PourFichier; // set up our callback procs, file
  PADeclencherDossier:=PourDossier; // folder
  Memo1.Clear;
  Memo2.Clear;
  ScruteDossier(Edit1.text,Edit2.Text,faAnyFile,CheckBox8.Checked); // execute
end;
end.

